import unittest
import numpy as np

from escape.core.array import *


class TestDoubleArrayMethods(unittest.TestCase):

    def test_array_creation(self):
        inp = np.linspace(1.0, 100.0, 1000, dtype=float)
        arr = double_array(inp, copy=False)
        self.assertEqual(len(inp), len(arr))
        for idx in range(0, len(arr)):
            self.assertEqual(inp[idx], arr[idx])
            
    def test_array_copy(self):
        inp = np.linspace(1.0, 100.0, 1000, dtype=float)
        arr = double_array(inp, copy=True)
        self.assertEqual(len(inp), len(arr))
        for idx in range(0, len(arr)):
            self.assertEqual(inp[idx], arr[idx])
        for idx in range(0, len(arr)):
            arr[idx] = len(arr) - idx
        for idx in range(0, len(arr)):
            self.assertNotEqual(arr[idx], inp[idx])
            
    def test_wrapping(self):
        inp = np.copy(np.linspace(1.0, 100.0, 1000, dtype=float))
        arr = double_array(inp, copy=False)
        self.assertEqual(len(inp), len(arr))
        for idx in range(0, len(arr)):
            self.assertEqual(inp[idx], arr[idx])
        for idx in range(0, len(arr)):
            arr[idx] = len(arr) - idx
        for idx in range(0, len(arr)):
            self.assertEqual(arr[idx], inp[idx])
            
    def test_numpy_copy(self):
        inp = np.linspace(1.0, 100.0, 1000, dtype=float)
        arr = double_array(inp, copy=True)
        nparr = np.array(arr)
        for idx in range(0, len(arr)):
            self.assertEqual(nparr[idx], arr[idx])
        for idx in range(0, len(arr)):
            nparr[idx] = len(nparr) - idx
        for idx in range(0, len(arr)):
            self.assertNotEqual(nparr[idx], arr[idx])
            
    def test_numpy_wrapping(self):
        inp = np.linspace(1.0, 100.0, 1000, dtype=float)
        arr = double_array(inp, copy=True)
        nparr = np.asarray(arr)
        for idx in range(0, len(arr)):
            self.assertEqual(nparr[idx], arr[idx])
        for idx in range(0, len(arr)):
            nparr[idx] = len(nparr) - idx
        for idx in range(0, len(arr)):
            self.assertEqual(nparr[idx], arr[idx])

            
class TestMaskArrayMethods(unittest.TestCase):

    def test_array_creation(self):
        inp = np.random.choice(a=[False, True], size=(1000,))
        arr = mask_array(inp, copy=False)
        self.assertEqual(len(inp), len(arr))
        for idx in range(0, len(arr)):
            self.assertEqual(inp[idx], arr[idx])
            
    def test_array_copy(self):
        inp = np.zeros(shape=1000)
        arr = mask_array(inp, copy=True)
        self.assertEqual(len(inp), len(arr))
        for idx in range(0, len(arr)):
            self.assertEqual(inp[idx], arr[idx])
        for idx in range(0, len(arr)):
            arr[idx] = True
        for idx in range(0, len(arr)):
            self.assertNotEqual(arr[idx], inp[idx])
            
    def test_wrapping(self):
        inp = np.random.choice(a=[False, True], size=(1000,))
        arr = mask_array(inp, copy=False)
        self.assertEqual(len(inp), len(arr))
        for idx in range(0, len(arr)):
            self.assertEqual(inp[idx], arr[idx])
        for idx in range(0, len(arr)):
            arr[idx] = False
        for idx in range(0, len(arr)):
            self.assertEqual(arr[idx], inp[idx])
            
    def test_numpy_copy(self):
        inp = np.zeros(shape=1000)
        arr = mask_array(inp, copy=True)
        nparr = np.array(arr)
        for idx in range(0, len(arr)):
            self.assertEqual(nparr[idx], arr[idx])
        for idx in range(0, len(arr)):
            nparr[idx] = True
        for idx in range(0, len(arr)):
            self.assertNotEqual(nparr[idx], arr[idx])
            
    def test_numpy_wrapping(self):
        inp = np.random.choice(a=[False, True], size=(1000,))
        arr = mask_array(inp, copy=True)
        nparr = np.asarray(arr)
        for idx in range(0, len(arr)):
            self.assertEqual(nparr[idx], arr[idx])
        for idx in range(0, len(arr)):
            nparr[idx] = True
        for idx in range(0, len(arr)):
            self.assertEqual(nparr[idx], arr[idx])

        
if __name__ == '__main__':
    unittest.main()
