import unittest
import numpy as np

from escape.core.data import *

class TestDataMethods(unittest.TestCase):

    def test_data_creation_without_errors(self):
        absc=np.linspace(1.0, 100.0, 1000, dtype=float)
        exp=np.linspace(100.0, 1.0, 1000, dtype=float)
        err=np.sqrt(exp)
        
        dt=data("Data", absc, exp, copy=False)
        
        _absc=dt.coordinates
        _exp=dt.intensities
        _err=dt.errors
        
        self.assertEqual(len(dt), len(absc))
        self.assertEqual(len(dt), len(exp))
        
        for idx in range(0, len(dt)):
            self.assertEqual(absc[idx], _absc[idx])
            self.assertEqual(exp[idx], _exp[idx])
            self.assertEqual(err[idx], _err[idx])
            
    def test_data_creation_with_errors(self):
        absc=np.linspace(1.0, 100.0, 1000, dtype=float)
        exp=np.linspace(100.0, 1.0, 1000, dtype=float)
        err=np.ones(shape=absc.shape)
        
        dt=data("Data", absc, exp, errors=err, copy=False)
        
        _absc=dt.coordinates
        _exp=dt.intensities
        _err=dt.errors
        
        self.assertEqual(len(dt), len(absc))
        self.assertEqual(len(dt), len(exp))
        
        for idx in range(0, len(dt)):
            self.assertEqual(absc[idx], _absc[idx])
            self.assertEqual(exp[idx], _exp[idx])
            self.assertEqual(err[idx], _err[idx])
            
    def test_data_creation_exception(self):
        absc=np.linspace(1.0, 100.0, 100, dtype=float)
        exp=np.linspace(100.0, 1.0, 1000, dtype=float)
        err=np.sqrt(exp)
        self.assertRaises(RuntimeError, 
                          lambda: data("Data", absc, exp, copy=False))
        
        
            
            
if __name__ == '__main__':
    unittest.main()
