import unittest
import numpy as np

from escape.core.objects import *


class TestFunctorMethods(unittest.TestCase):

    def test_identity_functor(self):
        x = var("X")
        
        f1 = x + 2
        
        self.assertAlmostEqual(f1.num_of_params, 0)
        self.assertAlmostEqual(f1(1.0), 3.0)
        
    def test_functor_math(self):
         
        x = var("x")
         
        f1 = x
        f2 = 10
         
        fp = f1 + f2
        fm = f1 - f2
        fmt = f1 * f2
        fd = f1 / f2
         
        self.assertAlmostEqual(fp(2.0), 12.0)
        self.assertAlmostEqual(fm(2.0), -8.0)
        self.assertAlmostEqual(fmt(2.0), 20.0)
        self.assertAlmostEqual(fd(2.0), 2.0 / 10.0)
        
    def test_functor_logic(self):
         
        x = var("X")
         
        f1 = x
        f2 = 0.5
         
        fg = f1 > f2
        fl = f1 < f2
        fe = f1 == f2
        fne = f1 != f2
        fge = f1 >= f2
        fle = f1 <= f2
         
        self.assertEqual(fg(1.0), True)
        self.assertEqual(fl(1.0), False)
        self.assertEqual(fe(1.0), False)
        self.assertEqual(fne(1.0), True)
        self.assertEqual(fge(1.0), True)
        self.assertEqual(fle(1.0), False)
      
    def test_functor_condition(self):
        x = var("X")
         
        f1 = x
        f2 = 2
        
        c=f1>f2

        f3 = conditional(c, func("", x, 22), func("", x, 65))
         
        self.assertAlmostEqual(f3(1.0), 65)
        self.assertAlmostEqual(f3(3.0), 22)
        
        
if __name__ == '__main__':
    unittest.main()
