import unittest
import numpy as np

from escape.core.objects import *


class TestParameterMethods(unittest.TestCase):

    def test_parameter_properties(self):
        p1 = par(name="parameter", value=10.0, userlim=None, 
              fixed=False,
              scale=1.0, limit_scale=0.5,
              units="arbitr.")
        
        self.assertEqual(p1.units, "arbitr.")
        self.assertEqual(p1.name, "parameter")
        self.assertAlmostEqual(p1.value, 10.0)
        self.assertAlmostEqual(p1.scale, 1.0)
        self.assertAlmostEqual(p1.min, 5.0)
        self.assertAlmostEqual(p1.max, 15.0)
        
        self.assertEqual(p1.fixed, False)
        self.assertEqual(p1.independent, True)
    
    def test_parameter_math(self):
        p1 = par("", value=5)
        p2 = par("", value=15)
        
        p_plus = p1 + p2
        p_minus = p1 - p2
        p_mult = p1 * p2
        p_div = p1 / p2
        
        self.assertAlmostEqual(p_plus.value, 5.0 + 15.0)
        self.assertAlmostEqual(p_minus.value, 5.0 - 15.0)
        self.assertAlmostEqual(p_mult.value, 5.0 * 15.0)
        self.assertAlmostEqual(p_div.value, 5.0 / 15.0)
        
        self.assertEqual(p1.independent, True)
        self.assertEqual(p1.independent, True)
        self.assertEqual(p_plus.independent, False)
        self.assertEqual(p_minus.independent, False)
        self.assertEqual(p_mult.independent, False)
        self.assertEqual(p_div.independent, False)
        
        self.assertEqual(p_plus.fixed, False)
        self.assertEqual(p_minus.fixed, False)
        self.assertEqual(p_mult.fixed, False)
        self.assertEqual(p_div.fixed, False)
            
        def func():
            p_div.value = 10

        self.assertRaises(RuntimeError, func)
       
    def test_parameter_condition(self):
        x = par("X", 10)
         
        c = conditional(x<5, 1, 0)
         
        self.assertAlmostEqual(c.value, 0)

if __name__ == '__main__':
    unittest.main()
